# #!/bin/bash

WORKINGDIR=$(pwd -P)
MYDIR=$(cd "$(dirname "$0")" && pwd -P)
MYFILENAME=$(basename "$0")

#
# Parse arguments
#
Usage() {
    echo >&2 "Usage: $0 [-o out_dir] R_VERSION"
    echo >&2 "  -o : output dir (defaults to ./R/)"
    echo >&2 "  R_VERSION: version of R to use (eg. 3.6)"
    exit 1
}

out_dir=
r_version=
while [[ "$1" == -* ]]; do
    if [[ $# -ge 2 && "$1" == "-o" ]]; then
        out_dir="$2"
        shift 2
    else
        Usage
    fi
done
if [[ $# -ne 1 ]]; then
    Usage
fi
r_version="$1"

if [[ -z $out_dir ]]; then
    out_dir="$WORKINGDIR/R"
    echo "R will be created at $out_dir"
fi

r_framework_dir="/Library/Frameworks/R.framework"

# Sets up variables based on the r version
# usage: $(setup_R_version)
setup_R_version() {
    echo "Setting up variables for R version $r_version"
    case $r_version in
        3.6)
            r_pkg_url='https://cran.r-project.org/bin/macosx/R-3.6.3.nn.pkg'
            r_version_dir="$r_framework_dir/Versions/$r_version"
            ressources_dir="$r_version_dir/Resources"
            libs=("libtcl8.6.dylib" "libtclstub8.6.a" "tcl8" "tcl8.6")
            include=("tcl.h" "tk.h")
            return;;
    esac
    echo "$r_version is not a supported R version"
    exit 1;
}

# Modifies the content of the wrapper to make R relocaltable
# Usage :$(modify_wrapper "$Wrapper_path")
modify_wrapper() {
    wrapper_file=$1
    wrapper_file_copy="$wrapper_file"_copy
    mv $wrapper_file $wrapper_file_copy

    default_r_home_dir='R_HOME_DIR=\/Library\/Frameworks\/R.framework\/Resources'
    new_r_home_dir='\
    MYDIR=\`dirname $0\`\
    R_HOME_DIR=\`cd $MYDIR\/.. \&\& pwd -P\`\
    \
    \# Used in Resources\/etc\/Renviron\
    export R_DOC_DIR\
    R_QPDF=${R_HOME_DIR}\/bin\/qpdf\
    '
    sed -e "s/$default_r_home_dir/$new_r_home_dir/g" -e 's/\/Library\/Frameworks\/R.framework\/Resources/$R_HOME_DIR/g' $wrapper_file_copy > $wrapper_file
    rm $wrapper_file_copy
    chmod u+x $wrapper_file
}

copy_tcltk() {
    for lib_filename in ${libs[@]}; do
        lib_path=/usr/local/lib/$lib_filename
        if [ -f $lib_path ]; then
            cp $lib_path $ressources_dir/lib
        elif [ -d $lib_path ]; then
            cp -r $lib_path $ressources_dir/lib
        else
            echo "cannot copy $lib_path"
        fi
    done
    for include_filename in ${include[@]}; do
        include_path=/usr/local/include/$include_filename
        if [ -f $include_path ]; then
            cp $include_path $ressources_dir/include
        elif [ -d $include_path ]; then
            cp -r $include_path $ressources_dir/include
        else
            echo "cannot copy $include_path"
        fi
    done
}

modify_Makeconf() {
    makeconf_file=$1
    makeconf_file_copy=${makeconf_file}_copy
    cp $makeconf_file $makeconf_file_copy
    sed -e "s/LIBR = -F\/Library\/Frameworks\/R.framework/\#\# LIBR = -F\/Library\/Frameworks\/R.framework/g" \
-e "s/TCLTK_CPPFLAGS = -I\/usr\/local\/include/TCLTK_CPPFLAGS = -I\${R_HOME}\/include/g" \
-e "s/TCLTK_LIBS = -L\/usr\/local\/lib/TCLTK_LIBS = -L\${R_HOME}\/lib/g" \
$makeconf_file_copy > $makeconf_file
    rm $makeconf_file_copy
}

# Modifies the content of the Rscript to make R relocaltable
# This is especially useful for R packages that compile from source and call Rscript without the RHOME set to the right value
# Usage :$(modify_Rscript "$Rscript_path")
modify_Rscript() {
    rscript_path=$1
    mv $rscript_path "${rscript_path}_real"
    echo '#!/bin/sh

MYDIR=`dirname $0`
DIRNAME=`basename $MYDIR`

if [ $DIRNAME == "bin" ]; then
    RHOME=$MYDIR/..
elif [ $DIRNAME == "Resources" ]; then
    RHOME=$MYDIR
else
    echo "Cannot set env variables... Dont know where we are"
    exit 1
fi

RHOME=$RHOME $MYDIR/Rscript_real "$@"
' > $rscript_path
    chmod +x $rscript_path
}

copy_relocatable_r_source_code() {
    script_file="$MYDIR/$MYFILENAME"
    if [ -f $script_file ]; then
        cp $script_file "$r_version_dir/$MYFILENAME"
    else
        echo "Could not provide source code for building this relocatable R: $script_file does not exist"
        exit 1
    fi
}

do_build_relocatable_R() {
    setup_R_version
    tmp_dir=$(mktemp -d -t relocatable-r)
    curl -o "$tmp_dir/R.pkg" $r_pkg_url
    sudo installer -pkg "$tmp_dir"/R.pkg -target /
    rm -r $tmp_dir
    
    copy_relocatable_r_source_code

    copy_tcltk
    modify_wrapper "$ressources_dir/bin/R"
    modify_Makeconf "$ressources_dir/etc/Makeconf"
    modify_Rscript "$ressources_dir/Rscript"
    modify_Rscript "$ressources_dir/bin/Rscript"

    if [[ ! -d $out_dir ]]; then
        mkdir $out_dir
    fi
    sudo mv -f $r_version_dir $out_dir
    sudo chmod -R 777 "$out_dir/$r_version"
    sudo tar -C $out_dir -czf "$out_dir/R-$r_version.tar.gz" "$r_version"

}

do_backup() {
    if [[ -d "$r_framework_dir" ]]; then
        sudo mv $r_framework_dir "${r_framework_dir}_backup"
        echo "backed up $r_framework_dir to ${r_framework_dir}_backup"
    fi
}

restore_backup() {
    sudo rm -rf $r_framework_dir
    if [[ -d "${r_framework_dir}_backup" ]]; then
        sudo mv "${r_framework_dir}_backup" $r_framework_dir
        echo "restored $r_framework_dir"
    fi
}

do_backup

do_build_relocatable_R || {
    restore_backup
    echo "failed to build relocatable R"
    exit 1;
}

restore_backup
exit 0
